@@ -1,4 +1,5 @@
 import os
+import sys
 import copy
 import shutil
 import collections
@@ -21,18 +22,19 @@ class Merger:
           nucmer_min_length_for_merges=4000,
           nucmer_breaklen=500,
           min_spades_circular_percent=95,
+          spades_kmers=None,
+          spades_use_first_success=False,
           ref_end_tolerance=15000,
           qry_end_tolerance=1000,
           verbose=False,
           threads=1,
           log_prefix='merge',
     ):
-        for f in [original_assembly, reassembly]:
-            if not os.path.exists(f):
-                raise Error('File not found:' + f)
+        if not os.path.exists(original_assembly):
+            raise Error('File not found:' + original_assembly)
 
         self.original_fasta = original_assembly
-        self.reassembly_fasta = reassembly
+        self.reassembly = circlator.assembly.Assembly(reassembly)
         self.reads = reads
         self.outprefix = outprefix
         self.nucmer_diagdiff = nucmer_diagdiff
@@ -41,6 +43,8 @@ class Merger:
         self.nucmer_min_length_for_merges = nucmer_min_length_for_merges
         self.nucmer_breaklen = nucmer_breaklen
         self.min_spades_circular_percent = min_spades_circular_percent
+        self.spades_kmers = spades_kmers
+        self.spades_use_first_success = spades_use_first_success
         self.ref_end_tolerance = ref_end_tolerance
         self.qry_end_tolerance = qry_end_tolerance
         self.verbose = verbose
@@ -48,9 +52,8 @@ class Merger:
         self.log_prefix = log_prefix
         self.merges = []
         self.original_contigs = {}
-        self.reassembly_contigs = {}
+        self.reassembly_contigs = self.reassembly.get_contigs()
         pyfastaq.tasks.file_to_dict(self.original_fasta, self.original_contigs)
-        pyfastaq.tasks.file_to_dict(self.reassembly_fasta, self.reassembly_contigs)
 
 
     def _run_nucmer(self, ref, qry, outfile):
@@ -222,8 +225,9 @@ class Merger:
         return False
 
 
-    def _get_possible_circular_ref_contigs(self, nucmer_hits):
+    def _get_possible_circular_ref_contigs(self, nucmer_hits, log_fh=None, log_outprefix=None):
         '''Returns a dict ref name => tuple(hit at start, hit at end) for each ref sequence in the hash nucmer_hits (each value is a list of nucmer hits)'''
+        writing_log_file = None not in [log_fh, log_outprefix]
         maybe_circular = {}
         all_nucmer_hits = []
         for l in nucmer_hits.values():
@@ -231,6 +235,9 @@ class Merger:
         nucmer_hits_by_qry = self._hits_hashed_by_query(all_nucmer_hits)
 
         for ref_name, list_of_hits in nucmer_hits.items():
+            if writing_log_file:
+                print(log_outprefix, ref_name, 'Checking ' + str(len(list_of_hits)) + ' nucmer hits', sep='\t', file=log_fh)
+
             longest_start_hit = self._get_longest_hit_at_ref_start(list_of_hits)
             longest_end_hit = self._get_longest_hit_at_ref_end(list_of_hits)
             if longest_start_hit == longest_end_hit:
@@ -247,6 +254,11 @@ class Merger:
               and longest_start_hit != longest_end_hit
               and self._hits_have_same_query(longest_start_hit, longest_end_hit)
             ):
+                if writing_log_file:
+                    print(log_outprefix, ref_name, 'potential pair of nucmer hits for circularization:', sep='\t', file=log_fh)
+                    print(log_outprefix, ref_name, '', longest_start_hit, sep='\t', file=log_fh)
+                    print(log_outprefix, ref_name, '', longest_end_hit, sep='\t', file=log_fh)
+
                 shortest_hit_length = self._min_qry_hit_length([longest_start_hit, longest_end_hit])
                 has_longer_hit = self._has_qry_hit_longer_than(
                     nucmer_hits_by_qry[longest_start_hit.qry_name],
@@ -254,16 +266,34 @@ class Merger:
                     hits_to_exclude={longest_start_hit, longest_end_hit}
                 )
 
-                if (not has_longer_hit) and self._can_circularise(longest_start_hit, longest_end_hit):
+                if writing_log_file and has_longer_hit:
+                    print(log_outprefix, ref_name, 'cannot use this pair because longer match was found', sep='\t', file=log_fh)
+
+                can_circularise = self._can_circularise(longest_start_hit, longest_end_hit)
+
+                if writing_log_file and not can_circularise:
+                    print(log_outprefix, ref_name, 'cannot use this pair because positions/orientations of matches no good', sep='\t', file=log_fh)
+
+                if (not has_longer_hit) and can_circularise:
+                    print(log_outprefix, ref_name, 'can use this pair of hits', sep='\t', file=log_fh)
                     maybe_circular[ref_name] = (longest_start_hit, longest_end_hit)
 
         return maybe_circular
 
 
-    def _remove_keys_from_dict_with_nonunique_values(self, d):
+    def _remove_keys_from_dict_with_nonunique_values(self, d, log_fh=None, log_outprefix=None):
         '''Returns a new dictionary, with keys from input dict removed if their value was not unique'''
         value_counts = collections.Counter(d.values())
-        return {key:d[key] for key in d if value_counts[d[key]] == 1}
+        new_d = {}
+        writing_log_file = None not in [log_fh, log_outprefix]
+
+        for key in d:
+            if value_counts[d[key]] == 1:
+                new_d[key] = d[key]
+            elif writing_log_file:
+                print(log_outprefix, 'Reject because non-unique:', d[key], sep='\t', file=log_fh)
+
+        return new_d
 
 
     def _make_circularised_contig(self, ref_start_hit, ref_end_hit):
@@ -277,11 +307,6 @@ class Merger:
         ref_start_coords = ref_start_hit.ref_coords()
         ref_end_coords = ref_end_hit.ref_coords()
 
-        if self.verbose:
-            print('[' + self.log_prefix + '] Using the following two nucmer hits to circularise', ref_name)
-            print('[' + self.log_prefix + ']    ', ref_start_hit)
-            print('[' + self.log_prefix + ']    ', ref_end_hit)
-
         if ref_start_coords.intersects(ref_end_coords):
             new_ctg = copy.copy(self.reassembly_contigs[qry_name])
             new_ctg.id = ref_name
@@ -302,20 +327,24 @@ class Merger:
 
 
     def _circularise_contigs(self, nucmer_hits):
+        log_fh = pyfastaq.utils.open_file_write(self.outprefix + '.circularise_details.log')
+        log_outprefix = '[merge circularise_details]'
+
         long_nucmer_hits = {}
         for name, hits in nucmer_hits.items():
             long_nucmer_hits[name] = [x for x in hits if x.hit_length_qry >= self.nucmer_min_length_for_merges]
-        to_circularise_with_nucmer = self._get_possible_circular_ref_contigs(long_nucmer_hits)
-        if self.verbose:
-            for name, hits in to_circularise_with_nucmer.items():
-                print('[' + self.log_prefix + ']', name, 'maybe circular:')
-                print('[' + self.log_prefix + ']    ', hits[0])
-                print('[' + self.log_prefix + ']    ', hits[1])
-
-        to_circularise_with_nucmer = self._remove_keys_from_dict_with_nonunique_values(to_circularise_with_nucmer)
+        to_circularise_with_nucmer = self._get_possible_circular_ref_contigs(long_nucmer_hits, log_fh=log_fh, log_outprefix=log_outprefix)
+        to_circularise_with_nucmer = self._remove_keys_from_dict_with_nonunique_values(to_circularise_with_nucmer, log_fh=log_fh, log_outprefix=log_outprefix)
         used_spades_contigs = set()
-        reassembly_fastg = self.reassembly_fasta[:-1] + 'g'
-        called_as_circular_by_spades = self._get_spades_circular_nodes(reassembly_fastg)
+        called_as_circular_by_spades = self.reassembly.circular_contigs()
+
+        if len(called_as_circular_by_spades):
+            circular_string = ','.join(sorted(called_as_circular_by_spades))
+        else:
+            circular_string = 'None'
+        print(log_outprefix, file=log_fh)
+        print(log_outprefix, '\tSPAdes reassembly contigs that are circular: ', circular_string, sep='', file=log_fh)
+
         fate_of_contigs = {
             x: set() for x in [
                 'repetitive_deleted',
@@ -325,35 +354,49 @@ class Merger:
         }
 
         for ref_name in self.original_contigs:
+            print(log_outprefix, sep='\t', file=log_fh)
+
             if ref_name in nucmer_hits:
-                new_contig, spades_contig = self._make_new_contig_from_nucmer_and_spades(ref_name, nucmer_hits[ref_name], called_as_circular_by_spades)
+                print(log_outprefix, ref_name, 'Trying to circularize. Has nucmer hits to check...', sep='\t', file=log_fh)
+                new_contig, spades_contig = self._make_new_contig_from_nucmer_and_spades(ref_name, nucmer_hits[ref_name], called_as_circular_by_spades, log_fh=log_fh, log_outprefix=log_outprefix)
 
                 if new_contig is None:
+                    print(log_outprefix, ref_name, 'Could not circularize using matches to SPAdes circular contigs', sep='\t', file=log_fh)
+
                     if ref_name in to_circularise_with_nucmer:
                         start_hit, end_hit = to_circularise_with_nucmer[ref_name]
                         assert start_hit.ref_name == end_hit.ref_name == ref_name
+                        print(log_outprefix, ref_name, 'Circularizing using this pair of nucmer matches to SPAdes contig:', sep='\t', file=log_fh)
+                        print(log_outprefix, '\t', ref_name, '\t\t', start_hit, sep='', file=log_fh)
+                        print(log_outprefix, '\t', ref_name, '\t\t', end_hit, sep='', file=log_fh)
                         new_contig = self._make_circularised_contig(start_hit, end_hit)
                         fate_of_contigs['circl_using_nucmer'].add(ref_name)
+                    else:
+                        print(log_outprefix, ref_name, 'Cannot circularize: no suitable nucmer hits', sep='\t', file=log_fh)
                 else:
                     assert new_contig.id == ref_name
                     assert spades_contig is not None
                     if spades_contig in used_spades_contigs:
-                        if self.verbose:
-                            print('[' + self.log_prefix + ']    ', ref_name, 'is circular, but duplicate sequence, so deleting it')
+                        print(log_outprefix, ref_name, 'Is circular, but duplicate sequence, so deleting it', sep='\t', file=log_fh)
                         fate_of_contigs['repetitive_deleted'].add(ref_name)
                     else:
                         self.original_contigs[new_contig.id] = new_contig
                         fate_of_contigs['circl_using_spades'].add(ref_name)
+                        print(log_outprefix, ref_name, 'Circularized using matches to SPAdes circular contigs', sep='\t', file=log_fh)
                         used_spades_contigs.add(spades_contig)
             else:
+                print(log_outprefix, ref_name, 'Cannot circularize: no nucmer hits', sep='\t', file=log_fh)
                 new_contig = None
 
 
-            if new_contig is not None:
+            if new_contig is None:
+                print(log_outprefix, ref_name, 'Circularized: no', sep='\t', file=log_fh)
+            else:
                 assert new_contig.id == ref_name
                 self.original_contigs[new_contig.id] = new_contig
+                print(log_outprefix, ref_name, 'Circularized: yes', sep='\t', file=log_fh)
 
-
+        pyfastaq.utils.close(log_fh)
         out_fasta = os.path.abspath(self.outprefix + '.fasta')
         fasta_fh = pyfastaq.utils.open_file_write(out_fasta)
         out_log = os.path.abspath(self.outprefix + '.circularise.log')
@@ -418,14 +461,18 @@ class Merger:
         return start_hit_ok and end_hit_ok
 
 
-    def _get_possible_query_bridging_contigs(self, nucmer_hits):
+    def _get_possible_query_bridging_contigs(self, nucmer_hits, log_fh=None, log_outprefix=None):
         '''Input is dict qry_name -> list of nucmer hits to that qry. Returns dict qry_name -> tuple(start hit, end hit)'''
         bridges = {}
+        writing_log_file = None not in [log_fh, log_outprefix]
 
         for qry_name, hits_to_qry in nucmer_hits.items():
             if len(hits_to_qry) < 2:
                 continue
 
+            if writing_log_file:
+                print(log_outprefix, '\t', qry_name, ': checking nucmer matches', sep='', file=log_fh)
+
             longest_start_hit = self._get_longest_hit_at_qry_start(hits_to_qry)
             longest_end_hit = self._get_longest_hit_at_qry_end(hits_to_qry)
 
@@ -434,8 +481,15 @@ class Merger:
               or longest_start_hit.ref_name == longest_end_hit.ref_name
               or self._hits_have_same_reference(longest_start_hit, longest_end_hit)
             ):
+                if writing_log_file:
+                    print(log_outprefix, '\t', qry_name, ': no potential pairs of hits to merge contigs', sep='', file=log_fh)
                 continue
 
+            if writing_log_file:
+                print(log_outprefix, '\t', qry_name, ': potential pair of hits to merge contigs...', sep='', file=log_fh)
+                print(log_outprefix, '\t', qry_name, ':    ', longest_start_hit, sep='', file=log_fh)
+                print(log_outprefix, '\t', qry_name, ':    ', longest_end_hit, sep='', file=log_fh)
+
             shortest_hit_length = self._min_qry_hit_length([longest_start_hit, longest_end_hit])
             has_longer_hit = self._has_qry_hit_longer_than(
                 hits_to_qry,
@@ -443,7 +497,17 @@ class Merger:
                 hits_to_exclude={longest_start_hit, longest_end_hit}
             )
 
-            if self._orientation_ok_to_bridge_contigs(longest_start_hit, longest_end_hit) and not has_longer_hit:
+            if has_longer_hit and writing_log_file:
+                print(log_outprefix, '\t', qry_name, ':    rejected - there is a longer hit to elsewhere', sep='', file=log_fh)
+
+            orientation_ok = self._orientation_ok_to_bridge_contigs(longest_start_hit, longest_end_hit)
+
+            if writing_log_file and not orientation_ok:
+                print(log_outprefix, '\t', qry_name, ':    rejected - orientation/distance from ends not correct to make a merge', sep='', file=log_fh)
+
+            if orientation_ok and not has_longer_hit:
+                if writing_log_file:
+                    print(log_outprefix, '\t', qry_name, ':    might be used - no longer hits elsewhere and orientation/distance to ends OK', sep='', file=log_fh)
                 bridges[qry_name] = (longest_start_hit, longest_end_hit)
 
         return bridges
@@ -478,7 +542,8 @@ class Merger:
         return bridges
 
 
-    def _merge_bridged_contig_pair(self, start_hit, end_hit, ref_contigs, qry_contigs):
+    def _merge_bridged_contig_pair(self, start_hit, end_hit, ref_contigs, qry_contigs, log_fh=None, log_outprefix=None):
+        writing_log_file = None not in [log_fh, log_outprefix]
         assert start_hit.qry_name == end_hit.qry_name
         assert start_hit.ref_name != end_hit.ref_name
         assert self._orientation_ok_to_bridge_contigs(start_hit, end_hit)
@@ -500,69 +565,117 @@ class Merger:
             tmp_seq.revcomp()
             end_seq = tmp_seq.seq
 
-        if self.verbose:
-            print('[' + self.log_prefix + '] Using the following two nucmer hits to merge contigs', start_hit.ref_name, end_hit.ref_name)
-            print('[' + self.log_prefix + ']    ', start_hit)
-            print('[' + self.log_prefix + ']    ', end_hit)
+        if writing_log_file:
+            print(log_outprefix, '\tMerging contigs ', start_hit.ref_name, ' and ', end_hit.ref_name, ' using these two hits:', sep='', file=log_fh)
+            print(log_outprefix, start_hit, sep='\t\t', file=log_fh)
+            print(log_outprefix, end_hit, sep='\t\t', file=log_fh)
+
         new_id = start_contig.id + '.' + end_contig.id
         new_contig = pyfastaq.sequences.Fasta(new_id, start_seq + bridge_seq + end_seq)
         self.merges.append([new_id, start_contig.id, end_contig.id])
         ref_contigs[new_contig.id] = new_contig
 
 
-    def _merge_all_bridged_contigs(self, nucmer_hits, ref_contigs, qry_contigs):
+    def _merge_all_bridged_contigs(self, nucmer_hits, ref_contigs, qry_contigs, log_fh=None, log_outprefix=None):
         '''Input is dict of nucmer_hits. Makes any possible contig merges.
            Returns True iff any merges were made'''
+        writing_log_file = None not in [log_fh, log_outprefix]
+
         if len(nucmer_hits) == 0:
+            if writing_log_file:
+                print(log_outprefix, 'No nucmer hits, so will not make any merges', sep='\t', file=log_fh)
             return
+
         all_nucmer_hits = []
         for l in nucmer_hits.values():
             all_nucmer_hits.extend(l)
         nucmer_hits_by_qry = self._hits_hashed_by_query(all_nucmer_hits)
-        bridges = self._get_possible_query_bridging_contigs(nucmer_hits_by_qry)
+        bridges = self._get_possible_query_bridging_contigs(nucmer_hits_by_qry, log_fh=log_fh, log_outprefix=log_outprefix)
+        if writing_log_file:
+            print(log_outprefix, '\tPotential contigs to use for merging: ', ' '.join(sorted(bridges.keys())), sep='', file=log_fh)
         bridges = self._filter_bridging_contigs(bridges)
+        if writing_log_file:
+            print(log_outprefix, '\tContigs to use for merging after uniqueness filtering: ', ' '.join(sorted(bridges.keys())), sep='', file=log_fh)
+
         merged = set()
         made_a_join = False
 
         for qry_name, (start_hit, end_hit) in bridges.items():
             if start_hit.ref_name in merged or end_hit.ref_name in merged:
                 continue
-            self._merge_bridged_contig_pair(start_hit, end_hit, ref_contigs, qry_contigs)
+            self._merge_bridged_contig_pair(start_hit, end_hit, ref_contigs, qry_contigs, log_fh=log_fh, log_outprefix=log_outprefix)
             merged.add(start_hit.ref_name)
             merged.add(end_hit.ref_name)
             made_a_join = True
 
+        if writing_log_file:
+            print(log_outprefix, '\tMade at least one contig join: ', made_a_join, sep='', file=log_fh)
+
         return made_a_join
 
 
+    def _index_fasta(self, infile):
+        fai = infile + '.fai'
+        if not os.path.exists(fai):
+            circlator.common.syscall('samtools faidx ' + infile, verbose=self.verbose)
+
+
+    def _write_act_files(self, ref_fasta, qry_fasta, coords_file, outprefix):
+        '''Writes crunch file and shell script to start up ACT, showing comparison of ref and qry'''
+        if self.verbose:
+            print('Making ACT files from', ref_fasta, qry_fasta, coords_file)
+        ref_fasta = os.path.relpath(ref_fasta)
+        qry_fasta = os.path.relpath(qry_fasta)
+        coords_file = os.path.relpath(coords_file)
+        outprefix = os.path.relpath(outprefix)
+        self._index_fasta(ref_fasta)
+        self._index_fasta(qry_fasta)
+        crunch_file = outprefix + '.crunch'
+        pymummer.coords_file.convert_to_msp_crunch(
+            coords_file,
+            crunch_file,
+            ref_fai=ref_fasta + '.fai',
+            qry_fai=qry_fasta + '.fai'
+        )
+
+        bash_script = outprefix + '.start_act.sh'
+        with open(bash_script, 'w') as f:
+            print('#!/usr/bin/env bash', file=f)
+            print('act', ref_fasta, crunch_file, qry_fasta, file=f)
+
+        pyfastaq.utils.syscall('chmod +x ' + bash_script)
+
+
     def _iterative_bridged_contig_pair_merge(self, outprefix):
         '''Iteratively merges contig pairs using bridging contigs from reassembly, until no more can be merged'''
         if self.reads is None:
-            return self.original_fasta, self.reassembly_fasta, None
+            return self.original_fasta, self.reassembly, None, None
 
+        log_file = outprefix + '.iterations.log'
+        log_fh = pyfastaq.utils.open_file_write(log_file)
         genome_fasta = self.original_fasta
-        reassembly_fasta = self.reassembly_fasta
-        nucmer_coords = outprefix + '.iter.0.coords'
+        nucmer_coords = outprefix + '.iter.1.coords'
         reads_to_map = self.reads
+        act_prefix = None
         made_a_join = True
-        iteration = 0
+        iteration = 1
 
         while made_a_join:
-            iteration += 1
-            self._run_nucmer(genome_fasta, reassembly_fasta, nucmer_coords)
+            this_log_prefix = '[' + self.log_prefix + ' iterative_merge ' + str(iteration) + ']'
+            print(this_log_prefix, '\tUsing nucmer matches from ', nucmer_coords, sep='', file=log_fh)
+            self._run_nucmer(genome_fasta, self.reassembly.contigs_fasta, nucmer_coords)
+            act_prefix = outprefix + '.iter.' + str(iteration)
+            print(this_log_prefix, '\tYou can view the nucmer matches with ACT using: ./', act_prefix, '.start_act.sh', sep='', file=log_fh)
+            self._write_act_files(genome_fasta, self.reassembly.contigs_fasta, nucmer_coords, act_prefix)
             nucmer_hits_by_ref = self._load_nucmer_hits(nucmer_coords)
-            made_a_join = self._merge_all_bridged_contigs(nucmer_hits_by_ref, self.original_contigs, self.reassembly_contigs)
+            made_a_join = self._merge_all_bridged_contigs(nucmer_hits_by_ref, self.original_contigs, self.reassembly_contigs, log_fh, this_log_prefix)
+            iteration += 1
 
             if made_a_join:
-                if self.verbose:
-                    print('[' + self.log_prefix + '] Contig merges were made. Running new read mapping and assembly')
-
+                print(this_log_prefix, '\tMade at least one merge. Remapping reads and reassembling',sep='', file=log_fh)
                 nucmer_coords = outprefix + '.iter.' + str(iteration) + '.coords'
                 genome_fasta = outprefix + '.iter.' + str(iteration) + '.merged.fasta'
-                reassembly_fasta = outprefix + '.iter.' + str(iteration) + '.reassembly.fasta'
-                reassembly_fastg = outprefix + '.iter.' + str(iteration) + '.reassembly.fastg'
                 self._contigs_dict_to_file(self.original_contigs, genome_fasta)
-                self._contigs_dict_to_file(self.reassembly_contigs, reassembly_fasta)
                 bam = outprefix + '.iter.' + str(iteration) + '.bam'
 
                 circlator.mapping.bwa_mem(
@@ -582,17 +695,18 @@ class Merger:
                     reads_prefix + '.fasta',
                     assembler_dir,
                     threads=self.threads,
-                    verbose=self.verbose
+                    verbose=self.verbose,
+                    spades_kmers=self.spades_kmers,
+                    spades_use_first_success=self.spades_use_first_success,
                 )
                 a.run()
-                os.rename(os.path.join(assembler_dir, 'contigs.fasta'), reassembly_fasta)
-                os.rename(os.path.join(assembler_dir, 'contigs.fastg'), reassembly_fastg)
-                shutil.rmtree(assembler_dir)
-                pyfastaq.tasks.file_to_dict(reassembly_fasta, self.reassembly_contigs)
-            elif iteration == 1 and self.verbose:
-                print('[' + self.log_prefix + '] No contig merges were made')
+                self.reassembly = circlator.assembly.Assembly(assembler_dir)
+                self.reassembly_contigs = self.reassembly.get_contigs()
+            elif iteration <= 2:
+                print(this_log_prefix, '\tNo contig merges were made',sep='', file=log_fh)
 
-        return genome_fasta, reassembly_fasta, nucmer_coords
+        pyfastaq.utils.close(log_fh)
+        return genome_fasta, nucmer_coords, act_prefix + '.start_act.sh'
 
 
     def _contigs_dict_to_file(self, contigs, fname):
@@ -622,23 +736,21 @@ class Merger:
         return found_fwd.intersection(found_rev)
 
 
-    def _make_new_contig_from_nucmer_and_spades(self, original_contig, hits, circular_spades):
+    def _make_new_contig_from_nucmer_and_spades(self, original_contig, hits, circular_spades, log_fh=None, log_outprefix=None):
         '''Tries to make new circularised contig from contig called original_contig. hits = list of nucmer hits, all with ref=original contg. circular_spades=set of query contig names that spades says are circular'''
+        writing_log_file = None not in [log_fh, log_outprefix]
         hits_to_circular_contigs = [x for x in hits if x.qry_name in circular_spades]
         if len(hits_to_circular_contigs) == 0:
+            if writing_log_file:
+                print(log_outprefix, original_contig, 'No matches to SPAdes circular contigs', sep='\t', file=log_fh)
             return None, None
 
-        if self.verbose:
-            print('[' + self.log_prefix + '] Hits to spades circular contig', hits[0].qry_name )
-            for hit in hits_to_circular_contigs:
-                print('[' + self.log_prefix + ']    ', hit)
-
         for hit in hits_to_circular_contigs:
+            print(log_outprefix, original_contig, 'Checking hit:', hit, sep='\t', file=log_fh)
             percent_query_covered = 100 * (hit.hit_length_qry / hit.qry_length)
-            if self.verbose:
-                print('[' + self.log_prefix + ']     percent query covered:', percent_query_covered, '...', hit)
 
             if self.min_spades_circular_percent <= percent_query_covered:
+                print(log_outprefix, '\t', original_contig, '\t\tHit is long enough. Percent of contig covered by hit is ', percent_query_covered, sep='', file=log_fh)
                 # the spades contig hit is long enough, but now check that
                 # the input contig is covered by hits from this spades contig
                 hit_intervals = [x.ref_coords() for x in hits_to_circular_contigs if x.qry_name == hit.qry_name]
@@ -646,13 +758,23 @@ class Merger:
                 if len(hit_intervals) > 0:
                     pyfastaq.intervals.merge_overlapping_in_list(hit_intervals)
                     percent_covered = 100 * pyfastaq.intervals.length_sum_from_list(hit_intervals) / hit.ref_length
-                    if self.verbose:
-                        print('[' + self.log_prefix + ']       reference bases covered by spades contig:', ', '.join([str(x) for x in hit_intervals]))
-                        print('[' + self.log_prefix + ']      ...which is', percent_covered, 'percent of', hit.ref_length, 'bases')
-                    if self.min_spades_circular_percent <= 100 * pyfastaq.intervals.length_sum_from_list(hit_intervals) / hit.ref_length:
-                        if self.verbose:
-                            print('[' + self.log_prefix + ']    ', hit.ref_name, 'is circular')
+
+                    if writing_log_file:
+                        print(log_outprefix, '\t', original_contig, '\t\treference bases covered by spades contig:', ', '.join([str(x) for x in hit_intervals]), sep='', file=log_fh)
+                        print(log_outprefix, '\t', original_contig, '\t\t   ... which is ', percent_covered, ' percent of ', hit.ref_length, ' bases', sep='', file=log_fh)
+
+                    if self.min_spades_circular_percent <= percent_covered:
+                        if writing_log_file:
+                            print(log_outprefix, original_contig, '\tUsing hit to call as circular (enough bases covered)', sep='\t', file=log_fh)
+
                         return pyfastaq.sequences.Fasta(original_contig, self.reassembly_contigs[hit.qry_name].seq), hit.qry_name
+                    elif writing_log_file:
+                        print(log_outprefix, original_contig, '\tNot using hit to call as circular (not enough bases covered)', sep='\t', file=log_fh)
+            else:
+                print(log_outprefix, original_contig, '\tNot using hit to call as circular (hit too short)', sep='\t', file=log_fh)
+
+        if writing_log_file:
+            print(log_outprefix, original_contig, 'No suitable matches to SPAdes circular contigs', sep='\t', file=log_fh)
 
         return None, None
 
@@ -668,14 +790,16 @@ class Merger:
 
     def run(self):
         out_log = os.path.abspath(self.outprefix + '.merge.log')
-        self.original_fasta, self.reassembly_fasta, nucmer_coords_file = self._iterative_bridged_contig_pair_merge(self.outprefix + '.merge')
+        self.original_fasta, nucmer_coords_file, act_script = self._iterative_bridged_contig_pair_merge(self.outprefix + '.merge')
         self._write_merge_log(self.outprefix + '.merge.log')
-        nucmer_coords = os.path.abspath(self.outprefix + '.coords')
+        nucmer_circularise_coords = os.path.abspath(self.outprefix + '.circularise.coords')
 
         if nucmer_coords_file is None:
-            self._run_nucmer(self.original_fasta, self.reassembly_fasta, nucmer_coords)
+            self._run_nucmer(self.original_fasta, self.reassembly.contigs_fasta, nucmer_circularise_coords)
+            self._write_act_files(self.original_fasta, self.reassembly.contigs_fasta, nucmer_circularise_coords, self.outprefix + '.circularise')
         else:
-            os.symlink(nucmer_coords_file, nucmer_coords)
+            os.symlink(nucmer_coords_file, nucmer_circularise_coords)
+            os.symlink(act_script, self.outprefix + '.circularise.start_act.sh')
 
-        nucmer_hits = self._load_nucmer_hits(nucmer_coords)
+        nucmer_hits = self._load_nucmer_hits(nucmer_circularise_coords)
         self._circularise_contigs(nucmer_hits)
